@@ -31,6 +31,31 @@ log = logging.getLogger("fs")
 # For long requests, we force a GIL release in the following interval
 GIL_RELEASE_INTERVAL = 0.05
 
+# ACL_ERRNO is the error code returned for requests that try
+# to modify or access extendeda attributes associated with ACL.
+# Since we currently don't know how to keep these in sync
+# with permission bits, we cannot support ACLs despite having
+# full support for extended attributes.
+#
+# What errno to use for ACL_ERRNO is a bit tricky. acl_set_fd(3)
+# returns ENOTSUP if the file system does not support ACLs. However,
+# this function uses the setxattr() syscall internally, and
+# setxattr(3) states that ENOTSUP means that the file system does not
+# support extended attributes at all. A test with btrfs mounted with
+# -o noacl shows that the actual errno returned by setxattr() is
+# EOPNOTSUPP. Also, some Python versions do not know about
+# errno.ENOTSUPP and errno(3) says that on Linux, EOPNOTSUPP and ENOTSUP
+# have the same value (despite this violating POSIX).
+#
+# All in all, the situation seems complicated, so we try to use
+# EOPNOTSUPP with a fallback on ENOTSUP just in case.
+
+if not hasattr(errno, 'EOPNOTSUPP'):
+    ACL_ERRNO = errno.ENOTSUP
+else:
+    ACL_ERRNO = errno.EOPNOTSUPP
+
+
 class Operations(llfuse.Operations):
     """A full-featured file system for online data storage
 
@@ -79,6 +104,9 @@ class Operations(llfuse.Operations):
         self.open_inodes = collections.defaultdict(lambda: 0)
         self.max_obj_size = max_obj_size
         self.cache = block_cache
+        
+        # Root inode is always open
+        self.open_inodes[llfuse.ROOT_INODE] += 1
 
     def destroy(self):
         self.forget(self.open_inodes.items())
@@ -177,6 +205,11 @@ class Operations(llfuse.Operations):
                 return self.extstat()
 
             raise llfuse.FUSEError(errno.EINVAL)
+        
+        # http://code.google.com/p/s3ql/issues/detail?id=385
+        elif name in (b'system.posix_acl_access',
+                      b'system.posix_acl_default'):
+            raise FUSEError(ACL_ERRNO)
 
         else:
             try:
@@ -217,6 +250,12 @@ class Operations(llfuse.Operations):
                 self.cache.max_size = pickle.loads(value)
             else:
                 raise llfuse.FUSEError(errno.EINVAL)
+            
+        # http://code.google.com/p/s3ql/issues/detail?id=385
+        elif name in (b'system.posix_acl_access',
+                      b'system.posix_acl_default'):
+            raise FUSEError(ACL_ERRNO)
+            
         else:
             if self.inodes[id_].locked:
                 raise FUSEError(errno.EPERM)
@@ -289,18 +328,20 @@ class Operations(llfuse.Operations):
             raise FUSEError(errno.EPERM)
 
         id0 = self.lookup(id_p0, name0).id
-        queue = [ id0 ]
+        queue = [ id0 ] # Directories that we still need to delete
         processed = 0 # Number of steps since last GIL release
         stamp = time.time() # Time of last GIL release
         gil_step = 250 # Approx. number of steps between GIL releases
-        while True:
-            found_subdirs = False
+        while queue: # For every directory
+            found_subdirs = False # Does current directory have subdirectories?
             id_p = queue.pop()
             for (name_id, id_) in self.db.query('SELECT name_id, inode FROM contents WHERE '
                                                 'parent_inode=?', (id_p,)):
 
                 if self.db.has_val('SELECT 1 FROM contents WHERE parent_inode=?', (id_,)):
                     if not found_subdirs:
+                        # When current directory has subdirectories, we must reinsert
+                        # it into queue
                         found_subdirs = True
                         queue.append(id_p)
                     queue.append(id_)
@@ -313,17 +354,10 @@ class Operations(llfuse.Operations):
 
                 processed += 1
                 if processed > gil_step:
-                    if not found_subdirs:
-                        found_subdirs = True
-                        queue.append(id_p)
+                    # Also reinsert current directory if we need to yield to other threads 
+                    queue.append(id_p)
                     break
 
-            if not queue:
-                if id_p0 in self.open_inodes:
-                    llfuse.invalidate_entry(id_p0, name0)
-                self._remove(id_p0, name0, id0, force=True)
-                break
-
             if processed > gil_step:
                 dt = time.time() - stamp
                 gil_step = max(int(gil_step * GIL_RELEASE_INTERVAL / dt), 250)
@@ -334,6 +368,13 @@ class Operations(llfuse.Operations):
                 log.debug('remove_tree(%d, %s): re-acquired lock', id_p0, name0)
                 stamp = time.time()
 
+        
+        if id_p0 in self.open_inodes:
+            log.debug('remove_tree(%d, %s): invalidate_entry(%d, %r)',
+                      id_p0, name0, id_p0, name0)                    
+            llfuse.invalidate_entry(id_p0, name0)
+        self._remove(id_p0, name0, id0, force=True)
+
         self.forget([(id0, 1)])
         log.debug('remove_tree(%d, %s): end', id_p0, name0)
 
@@ -709,8 +750,9 @@ class Operations(llfuse.Operations):
         """Handles FUSE setattr() requests"""
         if log.isEnabledFor(logging.DEBUG):
             log.debug('setattr(%d, %s): start', id_,
-                      [ getattr(attr, x) for x in attr.__slots__
-                       if getattr(attr, x) is not None ])
+                      ', '.join('%s=%r' % (x, getattr(attr, x)) 
+                                for x in attr.__slots__
+                                if getattr(attr, x) is not None ))
 
         inode = self.inodes[id_]
         timestamp = time.time()
